import logging
import json
import sys
from pathlib import Path
from typing import Dict, Any, Optional
from rich.logging import RichHandler
from rich.console import Console
from rich.traceback import install
import platform
from logging.handlers import RotatingFileHandler
from platformdirs import user_config_dir

# Константы
APP_NAME = "ai-ebash"
log_dir = Path(user_config_dir(APP_NAME)) / "logs"
log_dir.mkdir(parents=True, exist_ok=True)

# Заглушка для логгера по умолчанию (будет заменен)
logger = logging.getLogger('ai-ebash')

# Устанавливаем Rich для форматирования трейсбеков
install(show_locals=True)

# Преобразование строковых уровней в константы logging
def get_log_level(level_name: str) -> int:
    """Преобразует строковое имя уровня логирования в константу logging"""
    level_map = {
        "debug": logging.DEBUG,
        "info": logging.INFO,
        "warning": logging.WARNING,
        "error": logging.ERROR,
        "critical": logging.CRITICAL
    }
    return level_map.get(level_name.lower(), logging.INFO)

def configure_logger(config_data: Optional[Dict] = None) -> logging.Logger:
    """
    Настраивает и возвращает логгер с указанными параметрами.
    
    Args:
        config_data: Настройки логирования из конфигурации
    
    Returns:
        logging.Logger: Настроенный логгер
    """
    global logger
    
    # Значения по умолчанию
    log_level = logging.INFO
    console_level = logging.CRITICAL
    file_level = logging.DEBUG
    file_enabled = True
    
    # Применяем настройки из конфигурации, если они есть
    if config_data:      
        log_level = get_log_level(config_data.get('level', 'INFO'))
        console_level = get_log_level(config_data.get('console_level', 'INFO'))
        file_level = get_log_level(config_data.get('file_level', 'DEBUG'))
        file_enabled = config_data.get('file_enabled', True)
    
    # Создаем новый логгер
    logger = logging.getLogger('ai-ebash')
    logger.setLevel(log_level)
    
    # Очистка существующих обработчиков
    if logger.hasHandlers():
        logger.handlers.clear()
    
    # Консольный вывод
    console = Console()
    console_handler = RichHandler(
        console=console,
        rich_tracebacks=True,
        markup=True,
        show_path=False
    )
    console_handler.setLevel(console_level)
    logger.addHandler(console_handler)
    
    # Файловый вывод
    if file_enabled:
        file_formatter = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(filename)s:%(lineno)d - %(message)s'
        )
        file_handler = RotatingFileHandler(
            log_dir / "ai-ebash.log", 
            maxBytes=5*1024*1024,
            backupCount=3,
            encoding='utf-8'
        )
        file_handler.setLevel(file_level)
        file_handler.setFormatter(file_formatter)
        logger.addHandler(file_handler)
    
    # Логируем системную информацию при запуске
    logger.info(f"Запуск ai-ebash на {platform.system()} {platform.release()}")
    logger.debug(f"Python {platform.python_version()}, интерпретатор: {sys.executable}")
    logger.debug(f"Уровень логирования: консоль={console_level}, файл={file_level if file_enabled else 'отключено'}")
    
    return logger

# Первоначальная инициализация логгера с дефолтными настройками
logger = configure_logger(None)

def update_logger_config(config_data: dict):
    """
    Обновляет конфигурацию логгера на основе переданных настроек.
    Вызывается из settings.py после загрузки конфигурации.
    """
    global logger
    logger = configure_logger(config_data)
    logger.debug("Настройки логгера обновлены из конфигурационного файла")

# Вспомогательная функция для логирования времени выполнения
def log_execution_time(func):
    """Декоратор для логирования времени выполнения функции"""
    import time
    from functools import wraps
    
    @wraps(func)
    def wrapper(*args, **kwargs):
        start_time = time.time()
        logger.debug(f"Начало выполнения {func.__name__}")
        result = func(*args, **kwargs)
        execution_time = time.time() - start_time
        logger.debug(f"Функция {func.__name__} выполнена за {execution_time:.3f} сек")
        return result
    return wrapper


# openai_client.py
import json, requests
from typing import List

from .llm_interface import LLMClient
from aiebash.logger import logger
from rich.prompt import Prompt
from rich.console import Console

try:
    from .formatter_text import _format_api_key_display
except ImportError:
    # Для случаев, когда модуль запускается напрямую
    from formatter_text import _format_api_key_display


class DefaultClient(LLMClient):
    def __init__(self, model: str, api_url: str, api_key: str = None, timeout: int = 60):
        super().__init__(timeout=timeout)
        self.model = model
        self.api_url = api_url
        self.api_key = api_key
        logger.info(f"Инициализирован {self.__class__.__name__} с моделью {model}")
        logger.debug(f"API URL: {api_url}, timeout: {timeout}с")

    def configure_llm(self, console: Console) -> dict:
        """
        Настройка параметров OpenAI через прокси.
        Настраивается только модель и API ключ, URL фиксирован.
        """
        console.print("\n[bold]Настройка OpenAI через прокси:[/bold]")

        # Текущие значения
        current_model = getattr(self, 'model', 'gpt-4o-mini')
        current_api_key = getattr(self, 'api_key', '')

        # Настройка модели
        new_model = Prompt.ask("Модель", default=current_model)
        if not new_model.strip():
            new_model = current_model

        # Настройка API ключа
        new_api_key = Prompt.ask("API Key", default=current_api_key)

        # Возвращаем обновленные настройки
        return {
            "model": new_model,
            "api_key": new_api_key,
            "api_url": self.api_url  # URL остается фиксированным
        }

    def _send_chat(self, messages: List[dict]) -> str:
        logger.info(f"Отправка запроса к {self.model} через прокси")

        payload = {"model": self.model, "messages": messages}
        headers = {"Content-Type": "application/json"}
        if self.api_key:
            headers["Authorization"] = f"Bearer {self.api_key}"
            logger.debug(f"Используется API-ключ: {_format_api_key_display(self.api_key)}")
        else:
            logger.debug("API-ключ не предоставлен (используется публичный доступ, если поддерживается)")
        try:

            logger.debug(f"Выполняется POST-запрос к {self.api_url}")
            resp = requests.post(self.api_url, headers=headers, json=payload, timeout=self.timeout)

            status_code = resp.status_code
            logger.debug(f"Получен ответ, статус: {status_code}")

            resp.raise_for_status()
            data = resp.json()

            if "usage" in data:
                usage = data["usage"]
                logger.info(f"Использование токенов: prompt={usage.get('prompt_tokens', 0)}, "
                           f"completion={usage.get('completion_tokens', 0)}, "
                           f"total={usage.get('total_tokens', 0)}")

            if "choices" in data and data["choices"]:
                answer = data["choices"][0]["message"]["content"]
                logger.info(f"Получен ответ длиной {len(answer)} символов")
                return answer

            logger.error(f"Неожиданный формат ответа API: {json.dumps(data)[:200]}...")
            raise RuntimeError("Unexpected API response format")

        except requests.exceptions.HTTPError as e:
            logger.error(f"HTTP ошибка: {e}", exc_info=True)
            raise
        except requests.exceptions.ConnectionError as e:
            logger.error(f"Ошибка соединения: {e}", exc_info=True)
            raise
        except requests.exceptions.Timeout as e:
            logger.error(f"Таймаут соединения: {e}", exc_info=True)
            raise
        except requests.exceptions.RequestException as e:
            logger.error(f"Ошибка запроса: {e}", exc_info=True)
            raise
        except Exception as e:
            logger.error(f"Непредвиденная ошибка: {e}", exc_info=True)
            raise



import re
import platform


def _format_api_key_display(api_key: str) -> str:
    """Форматирует отображение API ключа для логирования"""
    if not api_key:
        return "(не задан)"
    elif len(api_key) <= 10:
        return api_key
    else:
        return f"{api_key[:5]}...{api_key[-5:]}"


def annotate_code_blocks(md_text):
    """
    Находит fenced code blocks с языком bash (для Linux) или bat (для Windows),
    собирает их содержимое в список и добавляет над каждым блоком строку-метку.
    Возвращает (annotated_md, list_of_code_strings).
    """
    code_blocks = []
    counter = 0
    
    # Определяем тип системы и соответствующий язык
    # is_windows = platform.system().lower() == "windows"
    # code_lang = "bat" if is_windows else "bash"
    
    # Формируем паттерн в зависимости от ОС
    # pattern = re.compile(r"```" + code_lang + r"[^\n]*\n(.*?)```", re.DOTALL | re.IGNORECASE)

    pattern = re.compile(r"```" + r"[^\n]*\n(.*?)```", re.DOTALL | re.IGNORECASE)


    def repl(m):
        nonlocal counter
        counter += 1
        code = m.group(1).rstrip("\n")
        code_blocks.append(code)
        # label = f"\n**Блок кода {code_lang} [#{counter}]**\n"
        # return label + f"```{code_lang}\n" + code + "\n```"
        label = f"\nБлок кода [#{counter}]\n"
        return label + f"```\n" + code + "\n```"

    annotated = pattern.sub(repl, md_text)
    return annotated, code_blocks


import requests
from rich.console import Console

console = Console()

def handle_connection_error(error: Exception):
    """
    Обрабатывает типовые ошибки интернет соединения и выводит их в консоль.
    """
    if isinstance(error, requests.exceptions.ConnectionError):
        console.print("[dim]Ошибка соединения: не удалось подключиться к серверу. Проверьте интернет.[/dim]")
    elif isinstance(error, requests.exceptions.Timeout):
        console.print("[dim]Ошибка: время ожидания соединения истекло.[/dim]")
    elif isinstance(error, requests.exceptions.HTTPError):
        if hasattr(error, 'response') and error.response is not None:
            status_code = error.response.status_code
            if status_code == 403:
                console.print("[dim]Ошибка 403: Доступ запрещён\nВозможные причины:\n-Превышен лимит запросов (попробуйте через некоторое время)\n-Не поддерживается ваш регион (используйте VPN)[/dim]")
            elif status_code == 404:
                console.print("[dim]Ошибка 404: Ресурс не найден.[/dim]")
            elif status_code == 429:
                console.print("[dim]Ошибка 429: Слишком много запросов. Превышен лимит API. Попробуйте изменить параметры запроса или уменьшить частоту запросов.[/dim]")
            elif status_code == 401:
                console.print("[dim]Ошибка 401: Отказ в авторизации.Проверьте свой ключ API_KEY. Для получения ключа обратитесь к поставщику API. [link=https://github.com/Vivatist/ai-bash]Как получить ключ?[/link][/dim]")

            else:
                console.print(f"[dim]HTTP ошибка: {status_code}[/dim]")
        else:
            console.print("[dim]HTTP ошибка: неизвестная причина[/dim]")
    else:
        console.print(f"[dim]Неизвестная ошибка соединения: {error}[/dim]")



#!/usr/bin/env python3
import sys
from pathlib import Path

# Добавляем parent (src) в sys.path для локального запуска
sys.path.insert(0, str(Path(__file__).resolve().parents[1]))

# Сначала импортируем настройки без импорта логгера
from aiebash.config_manager import config_manager

# Теперь импортируем и настраиваем логгер
from aiebash.logger import configure_logger

# Получаем настройки логирования и настраиваем логгер
logging_config = config_manager.get_logging_config()
logger = configure_logger(logging_config)

# Теперь продолжаем импорты остальных модулей
from rich.console import Console
from rich.markdown import Markdown
from aiebash.llm_factory import create_llm_client
from aiebash.arguments import parse_args, parser
from aiebash.chat import chat_loop


# === Считываем глобальные настройки ===
logger.info("Загрузка настроек...")
CONTEXT: str = config_manager.get_value("global", "context", "")
CURRENT_LLM: str = config_manager.get_value("global", "current_LLM", "openai_over_proxy")

logger.debug(f"Заданы настройки - Системный контекст: {'(пусто)' if not CONTEXT else CONTEXT[:30] + '...'}")
logger.debug(f"Заданы настройки - Текущий LLM: {CURRENT_LLM}")

# Настройки конкретного LLM (например, openai_over_proxy)
MODEL = config_manager.get_value("supported_LLMs", CURRENT_LLM, {}).get("model", "")
API_URL = config_manager.get_value("supported_LLMs", CURRENT_LLM, {}).get("api_url", "")
API_KEY = config_manager.get_value("supported_LLMs", CURRENT_LLM, {}).get("api_key", "")

logger.debug(f"Заданы настройки - Модель: {MODEL}")
logger.debug(f"Заданы настройки - API URL: {API_URL}")
logger.debug(f"Заданы настройки - API Key: {'(не задан)' if not API_KEY else f'{API_KEY[:5]}...{API_KEY[-5:] if len(API_KEY) > 10 else API_KEY}'}")


# === Инициализация клиента ===
logger.debug("Инициализация LLM клиента %s", CURRENT_LLM)
try:
    llm_client = create_llm_client(
        backend=CURRENT_LLM,
        model=MODEL,
        api_url=API_URL,
        api_key=API_KEY,
    )
except Exception as e:
    logger.error(f"Ошибка при создании LLM клиента: {e}", exc_info=True)
    sys.exit(1)


# === Основная логика ===
def main() -> None:

    console = Console()

    try:
        args = parse_args()
        logger.info("Разбор аргументов командной строки...")
        logger.debug(f"Полученные аргументы: dialog={args.dialog}, settings={args.settings}, prompt={args.prompt or '(пусто)'}")
        
        # Обработка режима настройки
        if args.settings:
            logger.info("Запуск конфигурационного режима")
            try:
                from aiebash.config_manager import run_configuration_dialog
                run_configuration_dialog()
                logger.info("Конфигурационный режим завершен")
                return 0
            except Exception as e:
                logger.error(f"Ошибка в режиме конфигурации: {e}", exc_info=True)
                return 1
        
        chat_mode: bool = args.dialog
        prompt: str = " ".join(args.prompt)

        if chat_mode:
            logger.info("Запуск в режиме диалога")
            try:
                chat_loop(console, llm_client, CONTEXT, prompt or None)
            except Exception as e:
                logger.error(f"Ошибка в режиме диалога: {e}", exc_info=True)
                return 1
        else:
            logger.info("Запуск в режиме одиночного запроса")
            if not prompt:
                logger.warning("Запрос не предоставлен, показываем справку")
                parser.print_help()
                return 1
                
            try:
                logger.debug(f"Отправка запроса: '{prompt[:50]}'...")
                answer: str = llm_client.send_prompt(prompt, system_context=CONTEXT)
            except Exception as e:
                return 1
            
            console.print(Markdown(answer))
            logger.info("Запрос успешно выполнен")

    except KeyboardInterrupt:
        logger.info("Программа прервана пользователем")
        return 130
    except Exception as e:
        logger.critical(f"Необработанная ошибка: {e}", exc_info=True)
        return 1
    
    logger.info("Программа завершена успешно")
    return 0


if __name__ == "__main__":
    sys.exit(main())




from aiebash.script_executor import run_code_block
from rich.console import Console

def run_code_selection(console: Console, code_blocks: list):
    """
    Интерактивный выбор и запуск bash-блоков.
    Вводите номер блока для выполнения, 0/q/exit — выход.
    """
    while True:
        try:
            choice = console.input("[blue]\nВведите номер блока для запуска (0 — выход): [/blue]").strip()
            if choice.lower() in ("0", "q", "exit"):
                break
            if not choice.isdigit():
                console.print("[yellow]Введите номер блока или 0 для выхода.[/yellow]")
                continue
            idx = int(choice)
            if not (1 <= idx <= len(code_blocks)):
                console.print(f"[yellow]Нет такого блока. Всего: {len(code_blocks)}.[/yellow]")
                continue
            run_code_block(console, code_blocks, idx)
        except (EOFError, KeyboardInterrupt):
            #console.print("\n")
            break



import sys
from pathlib import Path
import pytest
from unittest.mock import patch
from requests.exceptions import HTTPError
import importlib.util

# Импортируем src/aiebash/__main__.py как модуль
MAIN_PATH = Path(__file__).resolve().parents[0] / ".." / "__main__.py"
spec = importlib.util.spec_from_file_location("main_mod", str(MAIN_PATH.resolve()))
main_mod = importlib.util.module_from_spec(spec)
spec.loader.exec_module(main_mod)


def test_main_handles_http_error(monkeypatch):
    # Мокаем аргументы командной строки
    monkeypatch.setattr(main_mod, "parse_args", lambda: type("Args", (), {"run": False, "chat": False, "prompt": ["тест"]})())
    # Мокаем llm_client.send_prompt для выбрасывания HTTPError
    with patch.object(main_mod.llm_client, "send_prompt", side_effect=HTTPError("403 Client Error")):
        # Мокаем Console.print для отслеживания вывода
        with patch.object(main_mod.Console, "print") as mock_print:
            # main() не должен падать, просто завершиться
            main_mod.main()
            mock_print.assert_not_called()
            
def test_main_handles_connection_error(monkeypatch):
    import requests
    # Мокаем аргументы командной строки
    monkeypatch.setattr(main_mod, "parse_args", lambda: type("Args", (), {"run": False, "chat": False, "prompt": ["тест"]})())
    # Мокаем llm_client.send_prompt для выбрасывания ConnectionError
    with patch.object(main_mod.llm_client, "send_prompt", side_effect=requests.exceptions.ConnectionError("Max retries exceeded")):
        with patch.object(main_mod.Console, "print") as mock_print:
            # main() не должен падать, просто завершиться
            main_mod.main()
            mock_print.assert_not_called()


import argparse

# Создаем парсер как объект модуля
parser = argparse.ArgumentParser(
    prog="ai",
    description=(
        "Утилита для общения с нейросетью "
        "(OpenAI, HuggingFace, Ollama и др.) "
        "не покидая командной строки."
    ),
)

parser.add_argument(
    "-d",
    "--dialog",
    action="store_true",
    help=(
        "Режим диалога с возможностью выполнять блоки кода из ответа. "
        "Выход из диалога: exit, quit, выход или Ctrl+C."
    ),
)

parser.add_argument(
    "-s",
    "--settings",
    action="store_true",
    help="Запуск интерактивного режима настройки приложения.",
)

parser.add_argument(
    "prompt",
    nargs="*",
    help="Ваш запрос к ИИ.",
)


def parse_args() -> argparse.Namespace:
    """
    Разбор аргументов командной строки.
    """
    return parser.parse_args()



# llm_factory.py
from .llm_interface import LLMClient
from .openai_client import OpenAIClientOverProxy, OpenAIClient
from .default_client import DefaultClient
# сюда в будущем можно добавить HuggingFaceClient, OllamaClient и др.

def create_llm_client(backend: str, model: str, api_url: str, api_key: str = None, timeout: int = 60) -> LLMClient:
    backend = backend.lower().strip()
    if backend == "OpenAI over Proxy":
        return OpenAIClientOverProxy(model=model, api_url=api_url, api_key=api_key, timeout=timeout)
    elif backend == "OpenAI":
        return OpenAIClient(model=model, api_url=api_url, api_key=api_key, timeout=timeout)
    # elif backend == "ollama":
    #     return OllamaClient(...)
    else:
        try:
            return DefaultClient(model=model, api_url=api_url, api_key=api_key, timeout=timeout)
        except :
            raise ValueError(f"Не удается создать LLM клиент для: {backend}")



# llm_interface.py
from typing import List
import threading
from rich.console import Console
import threading
import time

from aiebash.error_handling import handle_connection_error
from aiebash.logger import log_execution_time


class LLMClient:
    def __init__(self, timeout: int = 60):
        self.timeout = timeout

    def run_progress(self, stop_spinner: threading.Event) -> None:
        """Визуальный индикатор работы ИИ с точечным спиннером.
        Пока stop_event не установлен, показывает "Аи печатает...".
        """
        console = Console()
        with console.status("[dim]Ai печатает...[/dim]", spinner="dots", spinner_style="dim"):
            while not stop_spinner.is_set():
                time.sleep(0.1)

        

    @log_execution_time # декоратор логирования времени исполнениения
    def send_chat(self, messages: List[dict]) -> str:
        """
        Отправляет сообщения в LLM, отображает прогресс-бар.
        Должен быть переопределён в наследнике.
        """
        stop_spinner = threading.Event()
        progress_thread = threading.Thread(target=self.run_progress, args=(stop_spinner,), daemon=True)
        progress_thread.start()
        
        result = None
        error = None
        
        try:
            result = self._send_chat(messages)
        except KeyboardInterrupt:
            raise
        except Exception as e:
            # Сохраняем ошибку, но НЕ обрабатываем её здесь
            error = e
        finally:
            # 1. Останавливаем поток индикатора
            stop_spinner.set()
            # 2. Ждем завершения потока индикатора
            progress_thread.join(timeout=1.0)
            # 3. Небольшая пауза для гарантированного обновления консоли
            time.sleep(0.1)
        
        # 4. Только ПОСЛЕ полной очистки индикатора обрабатываем ошибку
        if error:
            handle_connection_error(error)
            raise error
            
        return result


    def _send_chat(self, messages: List[dict]) -> str:
        """
        Реализация отправки запроса к LLM. Переопределяется в наследнике.
        """
        raise NotImplementedError

    def send_prompt(self, prompt: str, system_context: str = "") -> str:
        messages = []
        if system_context:
            messages.append({"role": "system", "content": system_context})
        messages.append({"role": "user", "content": prompt})
        return self.send_chat(messages)

    def configure_llm(self, console: Console) -> dict:
        """
        Интерактивная настройка параметров LLM.
        Возвращает словарь с обновленными настройками.
        Должен быть переопределён в наследнике.
        """
        raise NotImplementedError("Метод configure_llm должен быть реализован в наследнике")



# --- Top-level imports ---
from typing import List, Dict, Optional
from rich.console import Console
from rich.markdown import Markdown
from aiebash.formatter_text import annotate_code_blocks
from aiebash.script_executor import run_code_block
from aiebash.config_manager import config_manager
from aiebash.logger import logger


def _render_answer(console: Console, answer: str) -> List[str]:
    """Отрисовать ответ AI. При run_mode=True нумеруем bash-блоки, иначе просто показываем текст.
    Возвращает список bash-блоков (только при run_mode=True), иначе пустой список.
    """
    console.print("[bold blue]AI:[/bold blue]")
    logger.debug("Аннотирование и форматирование ответа от ИИ...")
    annotated_answer, code_blocks = annotate_code_blocks(answer)
    
    logger.debug(f"Текст ИИ в формате Markdown (с учетом аннотаций):\n{annotated_answer}")

    console.print(Markdown(annotated_answer))
    return code_blocks


def chat_loop(console: Console, llm_client, context: str, first_prompt: Optional[str]) -> None:
    """
    Запускает диалоговый режим с LLM.
    
    Args:
        console: Консоль для вывода
        llm_client: Клиент для взаимодействия с LLM
        context: Системный контекст для LLM
        first_prompt: Первый запрос пользователя (опционально)
    """
    logger.info("Запуск диалогового режима")
    messages: List[Dict[str, str]] = []
    
    if context:
        logger.debug(f"Установка системного контекста: {context[:50]}...")
        messages.append({"role": "system", "content": context})
    else:
        logger.debug("Системный контекст не задан")

    code_blocks: List[str] = []

    # Первый вопрос
    if first_prompt:
        logger.info(f"Обработка первого запроса: {first_prompt[:50]}...")
        messages.append({"role": "user", "content": first_prompt})
        
        try:
            logger.debug("Отправка первого запроса диалога")
            answer: str = llm_client.send_chat(messages)
            
            messages.append({"role": "assistant", "content": answer})
            code_blocks = _render_answer(console, answer)
            logger.info(f"Первый ответ отрендерен, найдено {len(code_blocks)} блоков кода")
        except Exception as e:
            logger.error(f"Ошибка при обработке первого запроса: {e}", exc_info=True)


    # Основной цикл
    logger.info("Запуск основного диалогового цикла")
    while True:
        try:
            # Вывод подсказки в зависимости от наличия блоков кода
            if len(code_blocks) > 0:
                logger.debug(f"Ожидание ввода пользователя (доступно {len(code_blocks)} блоков кода)")
                console.print("[dim]Введите следующий вопрос или номер блока кода для немедленного выполнения[/dim]")
            else:
                logger.debug("Ожидание ввода пользователя")
                console.print("[dim]Введите следующий вопрос[/dim]")
                
            # Ввод пользователя
            user_input: str = console.input("[bold green]Вы:[/bold green] ")
            stripped = user_input.strip()
            
            # Проверка на выход
            if stripped.lower() in ("exit", "quit", "выход"):
                logger.info("Пользователь запросил выход из диалога")
                break

            # Если введено число — попытка запуска блока
            if stripped.isdigit():
                idx = int(stripped)
                logger.debug(f"Пользователь запросил запуск блока #{idx}")
                
                if 1 <= idx <= len(code_blocks):
                    logger.info(f"Запуск блока кода #{idx}")
                    run_code_block(console, code_blocks, idx)
                else:
                    logger.warning(f"Запрошен несуществующий блок #{idx}, доступно {len(code_blocks)} блоков")
                    console.print("[yellow]Нет такого блока. Введите номер из списка или текстовый запрос.[/yellow]")
                continue  # Возвращаемся к вводу промпта

            # Обычное сообщение пользователя
            logger.info(f"Новый запрос пользователя: {user_input[:50]}...")
            messages.append({"role": "user", "content": user_input})
            
            try:
                logger.debug("Отправка запроса к LLM")
                answer = llm_client.send_chat(messages)
                
                messages.append({"role": "assistant", "content": answer})
                code_blocks = _render_answer(console, answer)
                logger.info(f"Ответ отрендерен, найдено {len(code_blocks)} блоков кода")
            except Exception as e:
                logger.error(f"Ошибка при обработке запроса: {e}", exc_info=True)
                console.print(f"[red]Ошибка при обработке запроса: {e}[/red]")

        except KeyboardInterrupt:
            logger.info("Диалог прерван пользователем (Ctrl+C)")
            console.print("\n")
            break
    
    logger.info("Диалоговый режим завершен")



#!/usr/bin/env python3
"""
Простой тест интерактивного режима настройки
"""

import sys
from pathlib import Path

# Добавляем src в путь
sys.path.insert(0, str(Path(__file__).resolve().parent / "src"))

from aiebash.config_manager import run_configuration_dialog

if __name__ == "__main__":
    print("Запуск теста интерактивной настройки...")
    try:
        run_configuration_dialog()
        print("Тест завершен успешно")
    except KeyboardInterrupt:
        print("\nТест прерван пользователем")
    except Exception as e:
        print(f"Ошибка в тесте: {e}")
        import traceback
        traceback.print_exc()


import subprocess
import platform
import tempfile
import os
from abc import ABC, abstractmethod
from rich.console import Console

from aiebash.logger import logger


# Абстрактный базовый класс для исполнителей команд
class CommandExecutor(ABC):
    """Базовый интерфейс для исполнителей команд разных ОС"""
    
    @abstractmethod
    def execute(self, code_block: str) -> subprocess.CompletedProcess:
        """
        Выполняет блок кода и возвращает результат
        
        Args:
            code_block (str): Блок кода для выполнения
            
        Returns:
            subprocess.CompletedProcess: Результат выполнения команды
        """
        pass


# Исполнитель команд для Linux
class LinuxCommandExecutor(CommandExecutor):
    """Исполнитель команд для Linux/Unix систем"""
    
    def execute(self, code_block: str) -> subprocess.CompletedProcess:
        """Выполняет bash-команды в Linux"""
        logger.debug(f"Выполнение bash-команды: {code_block[:80]}...")
        result = subprocess.run(
            code_block,
            shell=True,
            capture_output=True,
            text=True,
            encoding='utf-8'
        )
        logger.debug(f"Результат выполнения: код возврата {result.returncode}, "
                    f"stdout: {len(result.stdout)} байт, stderr: {len(result.stderr)} байт")
        return result


# Исполнитель команд для Windows
class WindowsCommandExecutor(CommandExecutor):
    """Исполнитель команд для Windows систем"""
    
    def execute(self, code_block: str) -> subprocess.CompletedProcess:
        """Выполняет bat-команды в Windows через временный файл"""
        # Предобработка кода для Windows
        code = code_block.replace('@echo off', '')
        code = code.replace('pause', 'rem pause')
        
        logger.debug(f"Подготовка Windows-команды: {code[:80]}...")
        
        # Создаем временный .bat файл
        fd, temp_path = tempfile.mkstemp(suffix='.bat')
        logger.debug(f"Создан временный файл: {temp_path}")
        
        try:
            with os.fdopen(fd, 'w') as f:
                f.write(code)
            
            # Запускаем с кодировкой консоли Windows
            logger.info(f"Выполнение команды из файла {temp_path}")
            result = subprocess.run(
                [temp_path],
                shell=True,
                capture_output=True,
                text=True,
                encoding='cp1251'  # Кириллическая кодировка для консоли Windows
            )
            logger.debug(f"Результат выполнения: код возврата {result.returncode}, "
                        f"stdout: {len(result.stdout)} байт, stderr: {len(result.stderr)} байт")
            return result
        except Exception as e:
            logger.error(f"Ошибка при выполнении Windows-команды: {e}", exc_info=True)
            raise
        finally:
            # Всегда удаляем временный файл
            try:
                os.unlink(temp_path)
                logger.debug(f"Временный файл {temp_path} удален")
            except Exception as e:
                logger.warning(f"Не удалось удалить временный файл {temp_path}: {e}")


# Фабрика для создания исполнителей команд
class CommandExecutorFactory:
    """Фабрика для создания исполнителей команд в зависимости от ОС"""
    
    @staticmethod
    def create_executor() -> CommandExecutor:
        """
        Создает исполнитель команд в зависимости от текущей ОС
        
        Returns:
            CommandExecutor: Соответствующий исполнитель для текущей ОС
        """
        system = platform.system().lower()
        if system == "windows":
            logger.info("Создание исполнителя команд для Windows")
            return WindowsCommandExecutor()
        else:
            logger.info(f"Создание исполнителя команд для {system} (используется LinuxCommandExecutor)")
            return LinuxCommandExecutor()


def run_code_block(console: Console, code_blocks: list, idx: int) -> None:
    """
    Печатает номер и содержимое блока, выполняет его и выводит результат.
    
    Args:
        console (Console): Консоль для вывода
        code_blocks (list): Список блоков кода
        idx (int): Индекс выполняемого блока
    """
    logger.info(f"Запуск блока кода #{idx}")
    
    # Проверяем корректность индекса
    if not (1 <= idx <= len(code_blocks)):
        logger.warning(f"Некорректный индекс блока: {idx}. Доступно блоков: {len(code_blocks)}")
        console.print(f"[yellow]Блок #{idx} не существует. Доступны блоки с 1 по {len(code_blocks)}.[/yellow]")
        return
    
    code = code_blocks[idx - 1]
    logger.debug(f"Содержимое блока #{idx}: {code[:100]}...")
    
    console.print(f"\n>>> Выполняем блок #{idx}:", style="blue")
    console.print(code)
    
    # Получаем исполнитель для текущей ОС
    try:
        executor = CommandExecutorFactory.create_executor()
        
        # Выполняем код через соответствующий исполнитель
        logger.debug("Начало выполнения блока кода...")
        process = executor.execute(code)
                
        # Выводим результаты
        if process.stdout:
            logger.debug(f"Получен stdout ({len(process.stdout)} символов)")
            console.print(f"[green]>>>:[/green]\n{process.stdout}")
        else:
            console.print("[green]>>> Нет вывода stdout[/green]")
            
        if process.stderr:
            console.print(f"[yellow]>>>Error:[/yellow]\n{process.stderr}")
        
        # Добавляем информацию о статусе выполнения
        exit_code = process.returncode
        logger.info(f"Блок #{idx} выполнен с кодом {exit_code}")
        console.print(f"[blue]>>> Код завершения: {exit_code}[/blue]")
            
    except Exception as e:
        logger.error(f"Ошибка выполнения блока #{idx}: {e}", exc_info=True)
        console.print(f"[yellow]Ошибка выполнения скрипта: {e}[/yellow]")